@@ -60,10 +60,13 @@ static int print_noqopt(struct qdisc_util *qu, FILE *f,
 	return 0;
 }
 
-static int parse_noqopt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n, const char *dev)
+static int parse_noqopt(struct qdisc_util *qu, int argc, char **argv,
+			struct nlmsghdr *n, const char *dev)
 {
 	if (argc) {
-		fprintf(stderr, "Unknown qdisc \"%s\", hence option \"%s\" is unparsable\n", qu->id, *argv);
+		fprintf(stderr,
+			"Unknown qdisc \"%s\", hence option \"%s\" is unparsable\n",
+			qu->id, *argv);
 		return -1;
 	}
 	return 0;
@@ -79,12 +82,15 @@ static int print_nofopt(struct filter_util *qu, FILE *f, struct rtattr *opt, __u
 	return 0;
 }
 
-static int parse_nofopt(struct filter_util *qu, char *fhandle, int argc, char **argv, struct nlmsghdr *n)
+static int parse_nofopt(struct filter_util *qu, char *fhandle,
+			int argc, char **argv, struct nlmsghdr *n)
 {
 	__u32 handle;
 
 	if (argc) {
-		fprintf(stderr, "Unknown filter \"%s\", hence option \"%s\" is unparsable\n", qu->id, *argv);
+		fprintf(stderr,
+			"Unknown filter \"%s\", hence option \"%s\" is unparsable\n",
+			qu->id, *argv);
 		return -1;
 	}
 	if (fhandle) {
@@ -186,23 +192,26 @@ noexist:
 
 static void usage(void)
 {
-	fprintf(stderr, "Usage: tc [ OPTIONS ] OBJECT { COMMAND | help }\n"
-			"       tc [-force] -batch filename\n"
-			"where  OBJECT := { qdisc | class | filter | action | monitor | exec }\n"
-	                "       OPTIONS := { -s[tatistics] | -d[etails] | -r[aw] | -p[retty] | -b[atch] [filename] | -n[etns] name |\n"
-			"                    -nm | -nam[es] | { -cf | -conf } path } | -j[son]\n");
+	fprintf(stderr,
+		"Usage: tc [ OPTIONS ] OBJECT { COMMAND | help }\n"
+		"       tc [-force] -batch filename\n"
+		"where  OBJECT := { qdisc | class | filter | action | monitor | exec }\n"
+		"       OPTIONS := { -V[ersion] | -s[tatistics] | -d[etails] | -r[aw] |\n"
+		"                    -j[son] | -p[retty] |\n"
+		"                    -b[atch] [filename] | -n[etns] name |\n"
+		"                    -nm | -nam[es] | { -cf | -conf } path }\n");
 }
 
-static int do_cmd(int argc, char **argv)
+static int do_cmd(int argc, char **argv, void *buf, size_t buflen)
 {
 	if (matches(*argv, "qdisc") == 0)
 		return do_qdisc(argc-1, argv+1);
 	if (matches(*argv, "class") == 0)
 		return do_class(argc-1, argv+1);
 	if (matches(*argv, "filter") == 0)
-		return do_filter(argc-1, argv+1);
+		return do_filter(argc-1, argv+1, buf, buflen);
 	if (matches(*argv, "actions") == 0)
-		return do_action(argc-1, argv+1);
+		return do_action(argc-1, argv+1, buf, buflen);
 	if (matches(*argv, "monitor") == 0)
 		return do_tcmonitor(argc-1, argv+1);
 	if (matches(*argv, "exec") == 0)
@@ -217,16 +226,116 @@ static int do_cmd(int argc, char **argv)
 	return -1;
 }
 
+#define TC_MAX_SUBC	10
+static bool batchsize_enabled(int argc, char *argv[])
+{
+	struct {
+		char *c;
+		char *subc[TC_MAX_SUBC];
+	} table[] = {
+		{ "filter", { "add", "delete", "change", "replace", NULL} },
+		{ "actions", { "add", "change", "replace", NULL} },
+		{ NULL },
+	}, *iter;
+	char *s;
+	int i;
+
+	if (argc < 2)
+		return false;
+
+	for (iter = table; iter->c; iter++) {
+		if (matches(argv[0], iter->c))
+			continue;
+		for (i = 0; i < TC_MAX_SUBC; i++) {
+			s = iter->subc[i];
+			if (s && matches(argv[1], s) == 0)
+				return true;
+		}
+	}
+
+	return false;
+}
+
+struct batch_buf {
+	struct batch_buf	*next;
+	char			buf[16420];	/* sizeof (struct nlmsghdr) +
+						   max(sizeof (struct tcmsg) +
+						   sizeof (struct tcamsg)) +
+						   MAX_MSG */
+};
+
+static struct batch_buf *get_batch_buf(struct batch_buf **pool,
+				       struct batch_buf **head,
+				       struct batch_buf **tail)
+{
+	struct batch_buf *buf;
+
+	if (*pool == NULL)
+		buf = calloc(1, sizeof(struct batch_buf));
+	else {
+		buf = *pool;
+		*pool = (*pool)->next;
+		memset(buf, 0, sizeof(struct batch_buf));
+	}
+
+	if (*head == NULL)
+		*head = *tail = buf;
+	else {
+		(*tail)->next = buf;
+		(*tail) = buf;
+	}
+
+	return buf;
+}
+
+static void put_batch_bufs(struct batch_buf **pool,
+			   struct batch_buf **head,
+			   struct batch_buf **tail)
+{
+	if (*head == NULL || *tail == NULL)
+		return;
+
+	if (*pool == NULL)
+		*pool = *head;
+	else {
+		(*tail)->next = *pool;
+		*pool = *head;
+	}
+	*head = NULL;
+	*tail = NULL;
+}
+
+static void free_batch_bufs(struct batch_buf **pool)
+{
+	struct batch_buf *buf;
+
+	for (buf = *pool; buf != NULL; buf = *pool) {
+		*pool = buf->next;
+		free(buf);
+	}
+	*pool = NULL;
+}
+
 static int batch(const char *name)
 {
-	char *line = NULL;
+	struct batch_buf *head = NULL, *tail = NULL, *buf_pool = NULL;
+	char *largv[100], *largv_next[100];
+	char *line, *line_next = NULL;
+	bool bs_enabled_next = false;
+	bool bs_enabled = false;
+	bool lastline = false;
+	int largc, largc_next;
+	bool bs_enabled_saved;
+	int batchsize = 0;
 	size_t len = 0;
 	int ret = 0;
+	bool send;
 
 	batch_mode = 1;
 	if (name && strcmp(name, "-") != 0) {
 		if (freopen(name, "r", stdin) == NULL) {
-			fprintf(stderr, "Cannot open file \"%s\" for reading: %s\n",
+			fprintf(stderr,
+				"Cannot open file \"%s\" for reading: %s\n",
 				name, strerror(errno));
 			return -1;
 		}
@@ -240,25 +349,95 @@ static int batch(const char *name)
 	}
 
 	cmdlineno = 0;
-	while (getcmdline(&line, &len, stdin) != -1) {
-		char *largv[100];
-		int largc;
+	if (getcmdline(&line, &len, stdin) == -1)
+		goto Exit;
+	largc = makeargs(line, largv, 100);
+	bs_enabled = batchsize_enabled(largc, largv);
+	bs_enabled_saved = bs_enabled;
+	do {
+		if (getcmdline(&line_next, &len, stdin) == -1)
+			lastline = true;
 
-		largc = makeargs(line, largv, 100);
-		if (largc == 0)
+		largc_next = makeargs(line_next, largv_next, 100);
+		bs_enabled_next = batchsize_enabled(largc_next, largv_next);
+		if (bs_enabled) {
+			struct batch_buf *buf;
+
+			buf = get_batch_buf(&buf_pool, &head, &tail);
+			if (!buf) {
+				fprintf(stderr,
+					"failed to allocate batch_buf\n");
+				return -1;
+			}
+			++batchsize;
+		}
+
+		/*
+		 * In batch mode, if we haven't accumulated enough commands
+		 * and this is not the last command and this command & next
+		 * command both support the batchsize feature, don't send the
+		 * message immediately.
+		 */
+		if (!lastline && bs_enabled && bs_enabled_next
+		    && batchsize != MSG_IOV_MAX)
+			send = false;
+		else
+			send = true;
+
+		line = line_next;
+		line_next = NULL;
+		len = 0;
+		bs_enabled_saved = bs_enabled;
+		bs_enabled = bs_enabled_next;
+		bs_enabled_next = false;
+
+		if (largc == 0) {
+			largc = largc_next;
+			memcpy(largv, largv_next, largc * sizeof(char *));
 			continue;	/* blank line */
+		}
 
-		if (do_cmd(largc, largv)) {
-			fprintf(stderr, "Command failed %s:%d\n", name, cmdlineno);
+		ret = do_cmd(largc, largv, tail == NULL ? NULL : tail->buf,
+			     tail == NULL ? 0 : sizeof(tail->buf));
+		if (ret != 0) {
+			fprintf(stderr, "Command failed %s:%d\n", name,
+				cmdlineno - 1);
 			ret = 1;
 			if (!force)
 				break;
 		}
-	}
-	if (line)
-		free(line);
+		largc = largc_next;
+		memcpy(largv, largv_next, largc * sizeof(char *));
 
+		if (send && bs_enabled_saved) {
+			struct iovec *iov, *iovs;
+			struct batch_buf *buf;
+			struct nlmsghdr *n;
+
+			iov = iovs = malloc(batchsize * sizeof(struct iovec));
+			for (buf = head; buf != NULL; buf = buf->next, ++iov) {
+				n = (struct nlmsghdr *)&buf->buf;
+				iov->iov_base = n;
+				iov->iov_len = n->nlmsg_len;
+			}
+
+			ret = rtnl_talk_iov(&rth, iovs, batchsize, NULL);
+			if (ret < 0) {
+				fprintf(stderr, "Command failed %s:%d\n", name,
+					cmdlineno - (batchsize + ret) - 1);
+				return 2;
+			}
+			put_batch_bufs(&buf_pool, &head, &tail);
+			batchsize = 0;
+			free(iovs);
+		}
+	} while (!lastline);
+
+	free_batch_bufs(&buf_pool);
+Exit:
+	free(line);
 	rtnl_close(&rth);
+
 	return ret;
 }
 
@@ -316,7 +495,9 @@ int main(int argc, char **argv)
 		} else if (matches(argv[1], "-json") == 0) {
 			++json;
 		} else {
-			fprintf(stderr, "Option \"%s\" is unknown, try \"tc -help\".\n", argv[1]);
+			fprintf(stderr,
+				"Option \"%s\" is unknown, try \"tc -help\".\n",
+				argv[1]);
 			return -1;
 		}
 		argc--;	argv++;
@@ -341,7 +522,7 @@ int main(int argc, char **argv)
 		goto Exit;
 	}
 
-	ret = do_cmd(argc-1, argv+1);
+	ret = do_cmd(argc-1, argv+1, NULL, 0);
 Exit:
 	rtnl_close(&rth);
 
